==> atmsupport-1.0.tar.gz: atmsupport-1.0/ats_sscop/user.c <==
/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/ats_sscop/user.c,v 1.4 2004/01/02 15:15:54 hbb Exp $
 */
#include <sys/types.h>
#include <sys/queue.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <limits.h>
#include <err.h>
#include <isc/eventlib.h>

#include <netnatm/unimsg.h>
#include <netnatm/saal/sscopdef.h>
#include <netnatm/saal/sscop.h>

#include "ats.h"
#include "test_sscop.h"

struct timer {
	evTimerID id;
	struct sscop *sscop;
	void (*func)(void *);
};

static struct sscop *stack;

/* default parameters */
static struct sscop_param sscop_default;

static u_int debug_flags;

/*
 * Set debugging
 */
static void
user_debug(u_int flags)
{
	debug_flags = flags;
}

/*
 * Reset the stack
 */
static void
user_reset(void)
{
	u_int mask = SSCOP_SET_ALLMASK;

	sscop_reset(stack);
	sscop_setparam(stack, &sscop_default, &mask);
	if (mask != 0)
		err_file("could not set default sscop params %x", mask);
}

static void
user_setparam(u_int mask, struct sscop_param *param)
{
	sscop_setparam(stack, param, &mask);
	if (mask != 0)
		err_file("could not set sscop params %x", mask);
}

static void
user_getparam(struct sscop_param *param)
{

	sscop_getparam(stack, param);
}

/*
 * Start the stack
 */
static void
user_start(void)
{

	sscop_setdebug(stack, debug_flags);
}

/*
 * timer helper
 */
static void
tfunc(evContext ctx __unused, void *uap, struct timespec due __unused,
    struct timespec inter __unused)
{
	struct timer *t = uap;

	t->func(t->sscop);
	free(t);
}


/*
 * Start an uni timer
 */
static void *
sscop_start_timer(struct sscop *sscop, void *arg, u_int msec,
    void (*func)(void *))
{
	struct timer *t;
	struct timespec due, now, ms;
	evContext *ctx = arg;

	if ((t = malloc(sizeof(*t))) == NULL)
		err(1, NULL);
	t->sscop = sscop;
	t->func = func;
	evInitID(&t->id);

	now = evNowTime();
	ms = evConsTime((time_t)(msec / 1000), (long)(msec % 1000) * 1000000);
	due = evAddTime(now, ms);

	if (evSetTimer(*ctx, tfunc, t, due, evConsTime(0, 0), &t->id))
		err(1, "cannot start timer");

	return (t);
}

/*
 * Stop an uni timer
 */
static void
sscop_stop_timer(struct sscop *sscop __unused, void *arg, void *tp)
{
	evContext *ctx = arg;
	struct timer *t = tp;

	evClearTimer(*ctx, t->id);
	free(t);
}

/*
 * output to the lower layer
 */
static void
sscop_send_lower(struct sscop *sscop __unused, void *arg __unused,
    struct uni_msg *msg)
{
	struct winput *ip;

	if ((ip = malloc(sizeof(*ip))) == NULL)
		err(1, NULL);

	ip->msg = msg;
	TAILQ_INSERT_TAIL(&lower_input_list, ip, link);
}

/*
 * output to the upper layer
 */
static void
sscop_send_upper(struct sscop *sscop __unused, void *arg __unused,
    enum sscop_aasig sig, struct uni_msg *msg, u_int sarg)
{
	struct winput *ip;

	if ((ip = malloc(sizeof(*ip))) == NULL)
		err(1, NULL);

	ip->msg = msg;
	ip->sig = sig;
	ip->arg1 = sarg;
	TAILQ_INSERT_TAIL(&upper_input_list, ip, link);
}

/*
 * output to the manage layer
 */
static void
sscop_send_manage(struct sscop *sscop __unused, void *arg __unused,
    enum sscop_maasig sig, struct uni_msg *msg, u_int sarg1, u_int sarg2)
{
	struct winput *ip;

	if ((ip = malloc(sizeof(*ip))) == NULL)
		err(1, NULL);

	ip->msg = msg;
	ip->sig = sig;
	ip->arg1 = sarg1;
	ip->arg2 = sarg2;
	TAILQ_INSERT_TAIL(&manage_input_list, ip, link);
}

/*
 * verbosity
 */
static void
sscop_verbose(struct sscop *sscop __unused, void *arg __unused,
    const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	fprintf(stderr, "\n");
}

static void
user_lower_to_sscop(struct uni_msg *msg)
{

	sscop_input(stack, msg);
}

static void
user_upper_to_sscop(u_int sig, u_int arg, struct uni_msg *msg)
{
	sscop_aasig(stack, sig, msg, arg);
}

static void
user_manage_to_sscop(u_int sig, u_int arg1 __unused, u_int arg2 __unused,
    struct uni_msg *msg)
{
	sscop_maasig(stack, sig, msg);
}

static const struct sscop_funcs sscop_funcs = {
	sscop_send_manage,
	sscop_send_upper,
	sscop_send_lower,
	sscop_verbose,
	sscop_start_timer,
	sscop_stop_timer
};

/*
 * Instantiate a user space stack
 */
static void
user_init(evContext *ctx)
{

	if ((stack = sscop_create(ctx, &sscop_funcs)) == NULL)
		err(1, "cannot create sscop instance");
	sscop_getparam(stack, &sscop_default);
}


const struct sscop_stack user_stack = {
	user_init,
	user_debug,
	user_upper_to_sscop,
	user_manage_to_sscop,
	user_lower_to_sscop,
	user_reset,
	user_start,
	user_setparam,
	user_getparam
};

==> atmsupport-1.4.tar.gz: atmsupport-1.4/ats_sscop/user.c <==
/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/ats_sscop/user.c,v 1.5 2004/07/08 13:33:32 brandt Exp $
 */
#include <sys/types.h>
#include <sys/queue.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <limits.h>

#include <netnatm/unimsg.h>
#include <netnatm/saal/sscopdef.h>
#include <netnatm/saal/sscop.h>

#include "ats.h"
#include "test_sscop.h"

static struct sscop *stack;

/* default parameters */
static struct sscop_param sscop_default;

static u_int debug_flags;

/*
 * Set debugging
 */
static void
user_debug(u_int flags)
{
	debug_flags = flags;
}

/*
 * Reset the stack
 */
static void
user_reset(void)
{
	u_int mask = SSCOP_SET_ALLMASK;

	sscop_reset(stack);
	sscop_setparam(stack, &sscop_default, &mask);
	if (mask != 0)
		err_file("could not set default sscop params %x", mask);
}

static void
user_setparam(u_int mask, struct sscop_param *param)
{
	sscop_setparam(stack, param, &mask);
	if (mask != 0)
		err_file("could not set sscop params %x", mask);
}

static void
user_getparam(struct sscop_param *param)
{

	sscop_getparam(stack, param);
}

/*
 * Start the stack
 */
static void
user_start(void)
{

	sscop_setdebug(stack, debug_flags);
}

/*
 * Start an uni timer
 */
static void *
sscop_start_timer(struct sscop *sscop, void *arg __unused, u_int msec,
    void (*func)(void *))
{

	return (ats_start_timer(msec, func, sscop));
}

/*
 * Stop an uni timer
 */
static void
sscop_stop_timer(struct sscop *sscop __unused, void *arg __unused, void *tp)
{

	ats_stop_timer(tp);
}

/*
 * output to the lower layer
 */
static void
sscop_send_lower(struct sscop *sscop __unused, void *arg __unused,
    struct uni_msg *msg)
{
	struct winput *ip;

	ip = ats_alloc(sizeof(*ip));
	ip->msg = msg;
	TAILQ_INSERT_TAIL(&lower_input_list, ip, link);
}

/*
 * output to the upper layer
 */
static void
sscop_send_upper(struct sscop *sscop __unused, void *arg __unused,
    enum sscop_aasig sig, struct uni_msg *msg, u_int sarg)
{
	struct winput *ip;

	ip = ats_alloc(sizeof(*ip));
	ip->msg = msg;
	ip->sig = sig;
	ip->arg1 = sarg;
	TAILQ_INSERT_TAIL(&upper_input_list, ip, link);
}

/*
 * output to the manage layer
 */
static void
sscop_send_manage(struct sscop *sscop __unused, void *arg __unused,
    enum sscop_maasig sig, struct uni_msg *msg, u_int sarg1, u_int sarg2)
{
	struct winput *ip;

	ip = ats_alloc(sizeof(*ip));
	ip->msg = msg;
	ip->sig = sig;
	ip->arg1 = sarg1;
	ip->arg2 = sarg2;
	TAILQ_INSERT_TAIL(&manage_input_list, ip, link);
}

/*
 * verbosity
 */
static void
sscop_verbose(struct sscop *sscop __unused, void *arg __unused,
    const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	vfprintf(stderr, fmt, ap);
	va_end(ap);
	fprintf(stderr, "\n");
}

static void
user_lower_to_sscop(struct uni_msg *msg)
{

	sscop_input(stack, msg);
}

static void
user_upper_to_sscop(u_int sig, u_int arg, struct uni_msg *msg)
{
	sscop_aasig(stack, sig, msg, arg);
}

static void
user_manage_to_sscop(u_int sig, u_int arg1 __unused, u_int arg2 __unused,
    struct uni_msg *msg)
{
	sscop_maasig(stack, sig, msg);
}

static const struct sscop_funcs sscop_funcs = {
	sscop_send_manage,
	sscop_send_upper,
	sscop_send_lower,
	sscop_verbose,
	sscop_start_timer,
	sscop_stop_timer
};

/*
 * Instantiate a user space stack
 */
static void
user_init(void)
{

	if ((stack = sscop_create(NULL, &sscop_funcs)) == NULL)
		err_file("cannot create sscop instance: %s", strerror(errno));
	sscop_getparam(stack, &sscop_default);
}


const struct sscop_stack user_stack = {
	user_init,
	user_debug,
	user_upper_to_sscop,
	user_manage_to_sscop,
	user_lower_to_sscop,
	user_reset,
	user_start,
	user_setparam,
	user_getparam
};
